import {
  ModalService, Modal, IButtonEvent, TextInput,
} from '@zeedhi/common';
import {
  Loader, Metadata, Http, I18n,
} from '@zeedhi/core';

export class Pla01300CargaModoPreparoController {
  private utilities = Loader.getInstance('Utilities');

  public modalCargaModoPrep!: Modal;
  
  private arrayPratosInicio: any[] = [];
  
  public openModalCargaModoPreparo() {
    this.createModalCargaModoPreparo().then(() => {
      this.modalCargaModoPrep.show();
    });
  }
  
  public async createModalCargaModoPreparo() {
    if (!this.modalCargaModoPrep) {
      this.modalCargaModoPrep = await ModalService.createFromJsonPost('',
        { jsonPath: 'planejamento/prato/modals/pla01303_carga_modo_preparo' });
    } else {
      await this.modalCargaModoPrep.destroy();
      this.modalCargaModoPrep = await ModalService.createFromJsonPost('',
        { jsonPath: 'planejamento/prato/modals/pla01303_carga_modo_preparo' });
    }
  }
  
  public closeModalCargaModoPreparo() {
    this.modalCargaModoPrep.destroy();
  }
  
  public fullscreen({ component }: IButtonEvent) {
    this.modalCargaModoPrep.fullscreen = !this.modalCargaModoPrep.fullscreen;
    component.iconName = (this.modalCargaModoPrep.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public retornaPratos() {
    Http.post('/cargaModoPrep/retornaQuantPratModoPrepDesatualizado', { requestType: 'Row' }).then((response) => {
      if (response && response.data && response.data.dataset.RETORNO) {
        const pratos = response.data.dataset.RETORNO;
        let campoQuantidadePratos = Metadata.getInstance<TextInput>('QUANT_PRATOS');
        if (pratos.length > 0) {
          this.arrayPratosInicio = JSON.parse(JSON.stringify(pratos));
          campoQuantidadePratos.value = pratos.length;
        } else if (pratos.length === 0) campoQuantidadePratos = pratos.length;
      }
    });
  }
  
  public async atualizaModoPreparo() {
    if (this.arrayPratosInicio.length > 0) this.updateModoPreparo(this.arrayPratosInicio);
    else this.utilities.showAlert('alert', 'Não existe pratos para realização da carga!', 'warning');
  }
  
  public updateModoPreparo(pratos: any[]) {
    const campoQtdPratos = Metadata.getInstance<TextInput>('QUANT_PRATOS');
    const percentPratAtualizados = Metadata.getInstance<TextInput>('PORCENTAGEM_REALIZADA');
    
    if (!pratos || pratos.length === 0) {
      if (!this.arrayPratosInicio || this.arrayPratosInicio.length === 0) {
        this.arrayPratosInicio = pratos;
      }
      percentPratAtualizados.value = '100%';
      campoQtdPratos.value = 0;
      return;
    }
    
    const qtdPratos = pratos.length;
    const tamanhoLote = Math.min(100, qtdPratos);
    const arrayParaUpdate = pratos.slice(0, tamanhoLote);
    const dados = { PRATOS: arrayParaUpdate };
    
    Http.post('/cargaModoPrep/updateModoReceita', { requestType: 'Row', row: dados }).then((response) => {
      pratos.splice(0, tamanhoLote);
      
      const quantidadePratosInicio = this.arrayPratosInicio.length;
      const quantidadeAtualizados = quantidadePratosInicio - pratos.length;
      const porcentagemAtualRealizada = Math.round((quantidadeAtualizados * 100) / quantidadePratosInicio);
      
      percentPratAtualizados.value = `${porcentagemAtualRealizada}%`;
      campoQtdPratos.value = pratos.length;
      
      if (percentPratAtualizados.value === '100%') {
        percentPratAtualizados.value = '0%';
      }
      
      if (response?.data?.dataset?.SUCESSO) {
        this.updateModoPreparo(pratos);
      }
      
      this.utilities.showAlert('alert', `${I18n.translate('Atualizado ')}
        ${percentPratAtualizados.value} ${I18n.translate(' do(s) modo(s) de preparo.')}`, 'success');
    }).catch(() => {
      this.utilities.showAlert('alert', 'Erro ao atualizar modos de preparo', 'error');
    }).finally(() => {
      this.arrayPratosInicio = [];
      this.utilities.showAlert('alert', 'Carga finalizada com sucesso', 'success');
    });
  }
}
